﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ContainerShip
{
    internal class MapsCollection
    {
        readonly Dictionary<string, MapWithSetShipGeneric<IDrawingObject, AbstractMap>> _mapStorages;        
        public List<string> Keys => _mapStorages.Keys.ToList();
       
        private readonly int _pictureWidth;
        
        private readonly int _pictureHeight;

        private readonly char separatorDict = '|';

        private readonly char separatorData = ';';

        public MapsCollection(int pictureWidth, int pictureHeight)
        {
            _mapStorages = new Dictionary<string, MapWithSetShipGeneric<IDrawingObject, AbstractMap>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        
        public void AddMap(string name, AbstractMap map)
        {
            if (_mapStorages.ContainsKey(name))
            {
                return;
            }
            MapWithSetShipGeneric<IDrawingObject, AbstractMap> newMap = new(_pictureWidth, _pictureHeight, map);
            _mapStorages.Add(name, newMap);
        }
        
        public void DelMap(string name)
        {
            if (_mapStorages.ContainsKey(name))
            {
                _mapStorages.Remove(name);
            }
        }
        
        public MapWithSetShipGeneric<IDrawingObject, AbstractMap> this[string
       ind]
        {
            get
            {
                if (_mapStorages.ContainsKey(ind))
                {
                    return _mapStorages[ind];
                }
                return null;
            }
        }
        public bool SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            using (StreamWriter sw = new(filename))
            {
                sw.Write($"MapsCollection{Environment.NewLine}");                
                foreach (var storage in _mapStorages)
                {
                    sw.Write($"{storage.Key}{separatorDict}{storage.Value.GetData(separatorDict, separatorData)}{Environment.NewLine}");
                }
            }
            return true;
        }

        public bool LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                return false;
            }
            using (StreamReader sr = new(filename))
            {
                string str;
                if ((str = sr.ReadLine()) == null || !str.Contains("MapsCollection"))
                {
                    return false;
                }
                _mapStorages.Clear();
                while((str = sr.ReadLine()) != null)
                {
                    var elem = str.Split(separatorDict);
                    AbstractMap map = null;
                    switch (elem[1])
                    {
                        case "SimpleMap":
                            map = new SimpleMap();
                            break;
                        case "IslandsMap":
                            map = new IslandsMap();
                            break;
                        case "RocksMap":
                            map = new RocksMap();
                            break;
                    }
                    _mapStorages.Add(elem[0], new MapWithSetShipGeneric<IDrawingObject, AbstractMap>(_pictureWidth, _pictureHeight, map));
                    _mapStorages[elem[0]].LoadData(elem[2].Split(separatorData, StringSplitOptions.RemoveEmptyEntries));
                }
            }
            return true;
        }
    }
}
